/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */
package org.ow2.petals.se.pojo.su;

import java.util.List;

import org.ow2.petals.component.framework.AbstractComponent;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.jbidescriptor.generated.Jbi;
import org.ow2.petals.component.framework.jbidescriptor.generated.Provides;
import org.ow2.petals.component.framework.su.ServiceEngineServiceUnitManager;
import org.ow2.petals.component.framework.su.ServiceUnitDataHandler;
import org.ow2.petals.component.framework.util.StringHelper;
import org.ow2.petals.se.pojo.Constants;
import org.ow2.petals.se.pojo.Pojo;
import org.ow2.petals.se.pojo.PojoComponent;

import static org.ow2.petals.component.framework.util.ClassLoaderUtil.createClassLoader;

/**
 * @author Frederic Gardes
 * 
 */
public class SUManager extends ServiceEngineServiceUnitManager {

    public SUManager(AbstractComponent engine) {
        super(engine);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.su.ServiceUnitListener#onSUDeployed
     * (java.lang.String, java.lang.String,
     * org.ow2.petals.component.framework.jbidescriptor.generated.Jbi)
     */
    @Override
    public void doDeploy(String serviceUnitName, String suRootPath, Jbi descriptor)
            throws PEtALSCDKException {

        if (descriptor.getServices().getConsumes().size() != 0) {
            throw new PEtALSCDKException("Consumes section are not supported by this component");
        }

        final List<Provides> providesList = descriptor.getServices().getProvides();
        if (providesList == null || providesList.size() != 1) {
            throw new PEtALSCDKException("Only one Provides section supported");
        }
        final Provides provides = providesList.get(0);
        if (provides != null) {
            final ServiceUnitDataHandler suDatahandler = getSUDataHandlerForService(provides);
            final ConfigurationExtensions extensions = suDatahandler
                    .getConfigurationExtensions(provides);
            final String className = extensions.get(Constants.CLASS_NAME);
            if (StringHelper.isNullOrEmpty(className)) {
                throw new PEtALSCDKException("Provides section must contains a POJO parameter '"
                        + Constants.CLASS_NAME + "'");
            }
        }
    }

    @Override
    public void doInit(String serviceUnitName, String suRootPath) throws PEtALSCDKException {
        final Jbi jbiDescriptor = getServiceUnitDataHandlers().get(serviceUnitName).getDescriptor();
        final Provides provides = jbiDescriptor.getServices().getProvides().get(0);
        final ServiceUnitDataHandler suDatahandler = getSUDataHandlerForService(provides);
        final ConfigurationExtensions extensions = suDatahandler
                .getConfigurationExtensions(provides);
        final String className = extensions.get(Constants.CLASS_NAME);
        Pojo pojo;
        try {
            pojo = createPojo(className, suDatahandler);
            pojo.init();
        } catch (Exception e) {
            throw new PEtALSCDKException(e);
        }
        ((PojoComponent) this.component).getPojos().put(provides.getEndpointName(), pojo);

    }

    /*
     * (non-Javadoc)
     * @see org.ow2.petals.component.framework.su.AbstractServiceUnitManager#doShutdown(java.lang.String)
     */
    @Override
    public void doShutdown(String serviceUnitName) throws PEtALSCDKException {
        final Jbi jbiDescriptor = getServiceUnitDataHandlers().get(serviceUnitName).getDescriptor();
        final Provides provides = jbiDescriptor.getServices().getProvides().get(0);
        try {
            ((PojoComponent) this.component).getPojos().get(provides.getEndpointName()).shutdown();
        } catch (Exception e) {
            throw new PEtALSCDKException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.su.ServiceUnitListener#onSUStarted
     * (java.lang.String)
     */
    @Override
    public void doStart(String serviceUnitName) throws PEtALSCDKException {
        final Jbi jbiDescriptor = getServiceUnitDataHandlers().get(serviceUnitName).getDescriptor();
        final Provides provides = jbiDescriptor.getServices().getProvides().get(0);
        try {
            ((PojoComponent) this.component).getPojos().get(provides.getEndpointName()).start();
        } catch (Exception e) {
            throw new PEtALSCDKException(e);
        }
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.su.ServiceUnitListener#onSUStopped
     * (java.lang.String)
     */
    @Override
    public void doStop(String serviceUnitName) throws PEtALSCDKException {
        final Jbi jbiDescriptor = getServiceUnitDataHandlers().get(serviceUnitName).getDescriptor();
        final Provides provides = jbiDescriptor.getServices().getProvides().get(0);
        try {
            ((PojoComponent) this.component).getPojos().get(provides.getEndpointName()).stop();
        } catch (Exception e) {
            throw new PEtALSCDKException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.su.ServiceUnitListener#onSUUndeployed
     * (java.lang.String, java.lang.String,
     * org.ow2.petals.component.framework.jbidescriptor.generated.Jbi)
     */
    @Override
    public void doUndeploy(String serviceUnitName) throws PEtALSCDKException {
        ((PojoComponent) this.component).getPojos().remove(serviceUnitName);
    }

    /**
     * Create the POJO.
     * 
     * @param className
     * @param suDatahandler
     * @return
     * @throws Exception
     */
    private Pojo createPojo(String className, ServiceUnitDataHandler suDatahandler)
            throws Exception {

        // create a classloader from the SU path
        ClassLoader classLoader = createClassLoader(suDatahandler.getInstallRoot(), this.getClass()
                .getClassLoader());

        // load the pojo class from the classloader
        Class<?> clazz = classLoader.loadClass(className);

        // create the pojo
        return new Pojo(clazz.newInstance(), classLoader, this.component.getContext(),
                this.component.getChannel(), logger);
    }

}
